#include <stdio.h>
#include "Ag34410.h"
#include <time.h>
#include <windows.h>

// Link to: <Program Files>\IVI Foundation\IVI\Lib\msc\Ag34410.lib

///
/// Agilent IVI-C Driver Example Program
/// 
/// Initializes the driver, reads a few Identity interface
/// properties, and checks the instrument error queue.
/// May include additional instrument specific functionality.
/// 
/// See driver help topic "Programming with the IVI-C Driver in Various Development Environments"
/// for additional programming information.
///
/// Runs in simulation mode without an instrument.
/// 

void main()
{
	ViStatus status;
	ViSession session;
	ViChar str[128];
	ViInt32 ErrorCode;
	ViChar ErrorMessage[256];
	ViBoolean simulate = VI_FALSE;
	ViInt32 actualPoints;
	ViReal64 dataArr[50000];
	ViInt16 i;
	FILE *fp = fopen("Data_Acquisition.txt","w");
	ViInt32 num_readings = 256+256;
	ViReal64 interval = 0.02;
	float seconds;
	clock_t start, end;

	// Edit resource and options as needed.  resource is ignored if option Simulate=true
	//char resource[] = "GPIB0::22::INSTR";
	char resource[] = "TCPIP0::172.16.7.136::INSTR";

	char options[]  = "QueryInstrStatus=true, Simulate=false, DriverSetup= Model=, Trace=false, Cache = true";

	ViBoolean idQuery = VI_TRUE;
	ViBoolean reset   = VI_TRUE;

	// Initialize the driver.  See driver help topic "Initializing the IVI-C Driver" for additional information
	status = Ag34410_InitWithOptions(resource, idQuery, reset, options, &session);
	if(status)
	{
		// Initialization failed
		Ag34410_GetError(session, &ErrorCode, 255, ErrorMessage);
		printf("** InitWithOptions() Error: %d, %s\n", ErrorCode, ErrorMessage);
		printf("\nDone - Press Enter to Exit");
		getchar();  
		return;
	}
	printf("Driver Initialized \n");

	// Read and output a few attributes
	/*status = Ag34410_GetAttributeViString(session, "", AG34410_ATTR_SPECIFIC_DRIVER_PREFIX, 127, str);
	printf("DRIVER_PREFIX:      %s\n", str);
	status = Ag34410_GetAttributeViString(session, "", AG34410_ATTR_SPECIFIC_DRIVER_REVISION, 127, str);
	printf("DRIVER_REVISION:    %s\n", str);
	status = Ag34410_GetAttributeViString(session, "", AG34410_ATTR_SPECIFIC_DRIVER_VENDOR, 127, str);
	printf("DRIVER_VENDOR:      %s\n", str);
	status = Ag34410_GetAttributeViString(session, "", AG34410_ATTR_SPECIFIC_DRIVER_DESCRIPTION, 127, str);
	printf("DRIVER_DESCRIPTION: %s\n", str);
	status = Ag34410_GetAttributeViString(session, "", AG34410_ATTR_INSTRUMENT_MODEL, 127, str);
	printf("INSTRUMENT_MODEL:   %s\n", str);
	status = Ag34410_GetAttributeViString(session, "", AG34410_ATTR_INSTRUMENT_FIRMWARE_REVISION, 127, str);
	printf("FIRMWARE_REVISION:  %s\n", str);
	status = Ag34410_GetAttributeViString(session, "", AG34410_ATTR_SERIAL_NUMBER, 127, str);
	printf("SERIAL_NUMBER:      %s\n", str);
	/*status = Ag34410_GetAttributeViBoolean(session, "", AG34410_ATTR_SIMULATE, &simulate);*/
	/*if (simulate == VI_TRUE)
		wprintf(L"\nSIMULATE:           True\n\n");
	else
		wprintf(L"SIMULATE:           False\n\n");*/


	//////////
	// Measure AC volts, auto range, 3 samples, 100uS/sample
	// Uses IviDmm functionality.
	//////////
	
	status = Ag34410_ConfigureMeasurement(session, AG34410_VAL_DC_VOLTS, -1, 0.00001);  // Range = -1 = auto range 
	//status = Ag34410_ConfigureMultiPoint(session, 1, 5, AG34410_VAL_IMMEDIATE, 0.0001 );

	//Sridutt's changes start here...
	printf("Waiting for %lf seconds\n",(interval*num_readings*2010)/1000);
    
	status = Ag34410_ConfigureTrigger(session, AG34410_VAL_EXTERNAL, -1);
	status = Ag34410_ConfigureMultiPoint(session, num_readings, 1, 1, 0.01);
    
	status = Ag34410_Initiate(session);
    printf("Status: %d, %x, %ld\n",status,status,status);


    // Fetch MultiPoint data
	start = clock();
	printf("Measuring DC volts, timer running.....\n");
	status = Ag34410_FetchMultiPoint(session, interval*num_readings*2010, 50000, dataArr, &actualPoints);
	end = clock();
	seconds = (float)(end - start)/CLOCKS_PER_SEC;
	
	printf("Fetch Multipoint time = %lf \n", seconds);

    //ViStatus Ag34410_FetchMultiPoint(ViSession Vi,ViInt32 MaxTimeMilliseconds,ViInt32 ArraySize,ViReal64[] ReadingArray,ViInt32* ActualPts);
    printf("Status: %d, %x, %ld\n",(int)status,status,(long int)status);

	//Sridutt's changes end here...

	/*status = Ag34410_Initiate(session);
	printf("Measuring DC volts...\n");

    // Fetch MultiPoint data
	status = Ag34410_FetchMultiPoint(session, 10000, 100, dataArr, &actualPoints);*/

	printf("Points: %d   \nData: ", actualPoints);
	for ( i=0; i<256; i++)
	{
		//printf("%lf, ",dataArr[i]);
		fprintf(fp,"%lf\n", dataArr[i]);
	}
	fclose(fp);
	printf("\n");

	// Check instrument for errors
	ErrorCode = 1;
	printf("\n");
	while(ErrorCode)
	{
		status = Ag34410_error_query( session, &ErrorCode, ErrorMessage);
		printf("error_query: %d, %s\n", ErrorCode, ErrorMessage);
	}

	// Close the driver
	status = Ag34410_close(session);
	if(status)
	{
		Ag34410_GetError(session, &ErrorCode, 255, ErrorMessage);
		printf("** Close() Error: %d, %s\n", ErrorCode, ErrorMessage);
	}
	else
		printf("Driver Closed \n");

	printf("\nPress any Key to Exit");
	getchar();  
}